import 'package:flutter/foundation.dart';

/// In this enumeration are enumerated all the currencies of the world,
/// according to [ISO 4217](https://www.iso.org/iso-4217-currency-codes.html).
///
enum Currency {
  /// UAE Dirham (United Arab Emirates)
  aed,

  /// Afghani (Afghanistan)
  afn,

  /// Lek (Albania)
  all,

  /// Armenian Dram (Armenia)
  amd,

  /// Netherlands Antillean Guilder (Curaçao, Sint Maarten (dutch part))
  ang,

  /// Kwanza (Angola)
  aoa,

  /// Argentine Peso (Argentina)
  ars,

  /// Australian Dollar (Australia, Christmas Island, Cocos (Keeling) Islands,
  /// Heard Island and McDonald Islands, Kiribati, Nauru, Norfolk Island,
  /// Tuvalu)
  aud,

  /// Aruban Florin (Aruba)
  awg,

  /// Azerbaijan Manat (Azerbaijan)
  azn,

  /// Convertible Mark (Bosnia and Herzegovina)
  bam,

  /// Barbados Dollar (Barbados)
  bbd,

  /// Taka (Bangladesh)
  bdt,

  /// Bulgarian Lev (Bulgaria)
  bgn,

  /// Bahraini Dinar (Bahrain)
  bhd,

  /// Burundi Franc (Burundi)
  bif,

  /// Bermudian Dollar (Bermuda)
  bmd,

  /// Brunei Dollar (Brunei Darussalam)
  bnd,

  /// Boliviano (Bolivia)
  bob,

  /// Mvdol (Bolivia, fund)
  bov,

  /// Brazilian Real (Brazil)
  brl,

  /// Bahamian Dollar (Bahamas)
  bsd,

  /// Ngultrum (Bhutan)
  btn,

  /// Pula (Botswana)
  bwp,

  /// Belarusian Ruble (Belarus)
  byn,

  /// Belize Dollar (Belize)
  bzd,

  /// Canadian Dollar (Canada)
  cad,

  /// Congolese Franc (Congo)
  cdf,

  /// WIR Euro (Switzerland, fund)
  che,

  /// Swiss Franc (Liechtenstein, Switzerland)
  chf,

  /// WIR Franc (Switzerland, fund)
  chw,

  /// Unidad de Fomento (Chile, fund)
  clf,

  /// Chilean Peso (Chile)
  clp,

  /// Yuan Renminbi (China)
  cny,

  /// Colombian Peso (Colombia)
  cop,

  /// Unidad de Valor Real (Colombia, fund)
  cou,

  /// Costa Rican Colon (Costa Rica)
  crc,

  /// Peso Convertible (Cuba)
  cuc,

  /// Cuban Peso (Cuba)
  cup,

  /// Cabo Verde Escudo (Cabo Verde)
  cve,

  /// Czech Koruna (Czechia)
  czk,

  /// Djibouti Franc (Djibouti)
  djf,

  /// Danish Krone (Denmark, Faroe Islands, Greenland)
  dkk,

  /// Dominican Peso (Dominican Republic)
  dop,

  /// Algerian Dinar (Algeria)
  dzd,

  /// Egyptian Pound (Egypt)
  egp,

  /// Nakfa (Eritrea)
  ern,

  /// Ethiopian Birr (Ethiopia)
  etb,

  /// Euro (Åland Islands, Andorra, Austria, Belgium, Cyprus, Estonia, European
  /// Union, Finland, France, French Guiana, French Southern Territories,
  /// Germany, Greece, Guadeloupe, Holy See, Ireland, Italy, Latvia, Lithuania,
  /// Luxembourg, Malta, Martinique, Mayotte, Monaco, Montenegro, Netherlands,
  /// Portugal, Réunion, Saint Barthélemy, Saint Martin (french part), Saint
  /// Pierre and Miquelon, San Marino, Slovakia, Slovenia, Spain)
  eur,

  /// Fiji Dollar (Fiji)
  fjd,

  /// Falkland Islands Pound (Falkland Islands (Malvinas))
  fkp,

  /// Pound Sterling (Guernsey, Isle of Man, Jersey, UNITED Kingdom of Great
  /// Britain and Northern Ireland)
  gbp,

  /// Lari (Georgia)
  gel,

  /// Ghana Cedi (Ghana)
  ghs,

  /// Gibraltar Pound (Gibraltar)
  gip,

  /// Dalasi (Gambia)
  gmd,

  /// Guinean Franc (Guinea)
  gnf,

  /// Quetzal (Guatemala)
  gtq,

  /// Guyana Dollar (Guyana)
  gyd,

  /// Hong Kong Dollar (Hong Kong)
  hkd,

  /// Lempira (Honduras)
  hnl,

  /// Kuna (Croatia)
  hrk,

  /// Gourde (Haiti)
  htg,

  /// Forint (Hungary)
  huf,

  /// Rupiah (Indonesia)
  idr,

  /// New Israeli Sheqel (Israel)
  ils,

  /// Indian Rupee (Bhutan, India)
  inr,

  /// Iraqi Dinar (Iraq)
  iqd,

  /// Iranian Rial (Iran)
  irr,

  /// Iceland Krona (Iceland)
  isk,

  /// Jamaican Dollar (Jamaica)
  jmd,

  /// Jordanian Dinar (Jordan)
  jod,

  /// Yen (Japan)
  jpy,

  /// Kenyan Shilling (Kenya)
  kes,

  /// Som (Kyrgyzstan)
  kgs,

  /// Riel (Cambodia)
  khr,

  /// Comorian Franc (Comoros)
  kmf,

  /// North Korean Won (North Korea)
  kpw,

  /// Won (South Korea)
  krw,

  /// Kuwaiti Dinar (Kuwait)
  kwd,

  /// Cayman Islands Dollar (Cayman Islands)
  kyd,

  /// Tenge (Kazakhstan)
  kzt,

  /// Lao Kip (Lao People’s Democratic Republic)
  lak,

  /// Lebanese Pound (Lebanon)
  lbp,

  /// Sri Lanka Rupee (Sri Lanka)
  lkr,

  /// Liberian Dollar (Liberia)
  lrd,

  /// Loti (Lesotho)
  lsl,

  /// Libyan Dinar (Libya)
  lyd,

  /// Moroccan Dirham (Morocco, Western Sahara)
  mad,

  /// Moldovan Leu (Moldova)
  mdl,

  /// Malagasy Ariary (Madagascar)
  mga,

  /// Denar (North Macedonia)
  mkd,

  /// Kyat (Myanmar)
  mmk,

  /// Tugrik (Mongolia)
  mnt,

  /// Pataca (Macao)
  mop,

  /// Ouguiya (Mauritania)
  mru,

  /// Mauritius Rupee (Mauritius)
  mur,

  /// Rufiyaa (Maldives)
  mvr,

  /// Malawi Kwacha (Malawi)
  mwk,

  /// Mexican Peso (Mexico)
  mxn,

  /// Mexican Unidad de Inversion (UDI) (Mexico, fund)
  mxv,

  /// Malaysian Ringgit (Malaysia)
  myr,

  /// Mozambique Metical (Mozambique)
  mzn,

  /// Namibia Dollar (Namibia)
  nad,

  /// Naira (Nigeria)
  ngn,

  /// Cordoba Oro (Nicaragua)
  nio,

  /// Norwegian Krone (Bouvet Island, Norway, Svalbard and Jan Mayen)
  nok,

  /// Nepalese Rupee (Nepal)
  npr,

  /// New Zealand Dollar (Cook Islands, New Zealand, Niue, Pitcairn, Tokelau)
  nzd,

  /// Rial Omani (Oman)
  omr,

  /// Balboa (Panama)
  pab,

  /// Sol (Peru)
  pen,

  /// Kina (Papua New Guinea)
  pgk,

  /// Philippine Peso (Philippines)
  php,

  /// Pakistan Rupee (Pakistan)
  pkr,

  /// Zloty (Poland)
  pln,

  /// Guarani (Paraguay)
  pyg,

  /// Qatari Rial (Qatar)
  qar,

  /// Romanian Leu (Romania)
  ron,

  /// Serbian Dinar (Serbia)
  rsd,

  /// Russian Ruble (Russian Federation)
  rub,

  /// Rwanda Franc (Rwanda)
  rwf,

  /// Saudi Riyal (Saudi Arabia)
  sar,

  /// Solomon Islands Dollar (Solomon Islands)
  sbd,

  /// Seychelles Rupee (Seychelles)
  scr,

  /// Sudanese Pound (Sudan)
  sdg,

  /// Swedish Krona (Sweden)
  sek,

  /// Singapore Dollar (Singapore)
  sgd,

  /// Saint Helena Pound (Saint Helena, Ascension and Tristan Da Cunha)
  shp,

  /// Leone (Sierra Leone)
  sll,

  /// Somali Shilling (Somalia)
  sos,

  /// Surinam Dollar (Suriname)
  srd,

  /// South Sudanese Pound (South Sudan)
  ssp,

  /// Dobra (Sao Tome and Principe)
  stn,

  /// El Salvador Colon (El Salvador)
  svc,

  /// Syrian Pound (Syrian Arab Republic)
  syp,

  /// Lilangeni (Eswatini)
  szl,

  /// Baht (Thailand)
  thb,

  /// Somoni (Tajikistan)
  tjs,

  /// Turkmenistan New Manat (Turkmenistan)
  tmt,

  /// Tunisian Dinar (Tunisia)
  tnd,

  /// Pa’anga (Tonga)
  top,

  /// Turkish Lira (Turkey)
  try_,

  /// Trinidad and Tobago Dollar (Trinidad and Tobago)
  ttd,

  /// New Taiwan Dollar (Taiwan)
  twd,

  /// Tanzanian Shilling (Tanzania)
  tzs,

  /// Hryvnia (Ukraine)
  uah,

  /// Uganda Shilling (Uganda)
  ugx,

  /// US Dollar (American Samoa, Bonaire, Sint Eustatius and Saba, British
  /// Indian Ocean Territory, Ecuador, El Salvador, Guam, Haiti, Marshall
  /// Islands, Micronesia, Northern Mariana Islands, Palau, Panama, Puerto Rico,
  /// Timor-Leste, Turks and Caicos Islands, United States Minor Outlying
  /// Islands, UNITED States of America, Virgin Islands (british), Virgin
  /// Islands (U.S.))
  usd,

  /// US Dollar (Next day) (United States of America, fund)
  usn,

  /// Uruguay Peso en Unidades Indexadas (UI) (Uruguay, fund)
  uyi,

  /// Peso Uruguayo (Uruguay)
  uyu,

  /// Unidad Previsional (Uruguay)
  uyw,

  /// Uzbekistan Sum (Uzbekistan)
  uzs,

  /// Bolívar Soberano (Venezuela)
  ves,

  /// Dong (Viet Nam)
  vnd,

  /// Vatu (Vanuatu)
  vuv,

  /// Tala (Samoa)
  wst,

  /// CFA Franc BEAC (Cameroon, Central African Republic, Chad, Congo,
  /// Equatorial Guinea, Gabon)
  xaf,

  /// Silver (ZZ11_Silver)
  xag,

  /// Gold (ZZ08_Gold)
  xau,

  /// Bond Markets Unit European Composite Unit (EURCO) (ZZ01_Bond Markets Unit
  /// European_EURCO)
  xba,

  /// Bond Markets Unit European Monetary Unit (E.M.U.-6) (ZZ02_Bond Markets
  /// Unit European_EMU-6)
  xbb,

  /// Bond Markets Unit European Unit of Account 9 (E.U.A.-9) (ZZ03_Bond Markets
  /// Unit European_EUA-9)
  xbc,

  /// Bond Markets Unit European Unit of Account 17 (E.U.A.-17) (ZZ04_Bond
  /// Markets Unit European_EUA-17)
  xbd,

  /// East Caribbean Dollar (Anguilla, Antigua and Barbuda, Dominica, Grenada,
  /// Montserrat, Saint Kitts and Nevis, Saint Lucia, Saint Vincent and The
  /// Grenadines)
  xcd,

  /// SDR (Special Drawing Right) (International Monetary Fund (IMF))
  xdr,

  /// CFA Franc BCEAO (Benin, Burkina Faso, Côte d’Ivoire, Guinea-Bissau, Mali,
  /// Niger, Senegal, Togo)
  xof,

  /// Palladium (ZZ09_Palladium)
  xpd,

  /// CFP Franc (French Polynesia, New Caledonia, Wallis and Futuna)
  xpf,

  /// Platinum (ZZ10_Platinum)
  xpt,

  /// Sucre (Sistema Unitario de Compensacion Regional de Pagos “SUCRE”)
  xsu,

  /// Codes specifically reserved for testing purposes (ZZ06_Testing_Code)
  xts,

  /// ADB Unit of Account (Member Countries of The African Development Bank
  /// Group)
  xua,

  /// The codes assigned for transactions where no currency is involved
  /// (ZZ07_No_Currency)
  xxx,

  /// Yemeni Rial (Yemen)
  yer,

  /// Rand (Lesotho, Namibia, South Africa)
  zar,

  /// Zambian Kwacha (Zambia)
  zmw,

  /// Zimbabwe Dollar (Zimbabwe)
  zwl,
}

/// This extension provides useful tools for `Currency` enumeration values.
///
extension CurrencyX on Currency {
  /// Gets the alphabetic code for this currency.
  ///
  String get alphabetic {
    final String description = describeEnum(this);

    final RegExp regExp = RegExp('([A-Za-z])+');
    final Iterable<Match> matches = regExp.allMatches(description);

    final onlyLettersBuffer = StringBuffer();
    for (final Match match in matches) onlyLettersBuffer.write(match.group(0));

    return '$onlyLettersBuffer'.toUpperCase();
  }

  /// Gets the number of decimal digits the currency’s representation requires.
  /// In the case that it does not apply, it returns `null`.
  ///
  int? get exponent {
    switch (this) {
      case Currency.aed:
        return 2;
      case Currency.afn:
        return 2;
      case Currency.all:
        return 2;
      case Currency.amd:
        return 2;
      case Currency.ang:
        return 2;
      case Currency.aoa:
        return 2;
      case Currency.ars:
        return 2;
      case Currency.aud:
        return 2;
      case Currency.awg:
        return 2;
      case Currency.azn:
        return 2;
      case Currency.bam:
        return 2;
      case Currency.bbd:
        return 2;
      case Currency.bdt:
        return 2;
      case Currency.bgn:
        return 2;
      case Currency.bhd:
        return 3;
      case Currency.bif:
        return 0;
      case Currency.bmd:
        return 2;
      case Currency.bnd:
        return 2;
      case Currency.bob:
        return 2;
      case Currency.bov:
        return 2;
      case Currency.brl:
        return 2;
      case Currency.bsd:
        return 2;
      case Currency.btn:
        return 2;
      case Currency.bwp:
        return 2;
      case Currency.byn:
        return 2;
      case Currency.bzd:
        return 2;
      case Currency.cad:
        return 2;
      case Currency.cdf:
        return 2;
      case Currency.che:
        return 2;
      case Currency.chf:
        return 2;
      case Currency.chw:
        return 2;
      case Currency.clf:
        return 4;
      case Currency.clp:
        return 0;
      case Currency.cny:
        return 2;
      case Currency.cop:
        return 2;
      case Currency.cou:
        return 2;
      case Currency.crc:
        return 2;
      case Currency.cuc:
        return 2;
      case Currency.cup:
        return 2;
      case Currency.cve:
        return 2;
      case Currency.czk:
        return 2;
      case Currency.djf:
        return 0;
      case Currency.dkk:
        return 2;
      case Currency.dop:
        return 2;
      case Currency.dzd:
        return 2;
      case Currency.egp:
        return 2;
      case Currency.ern:
        return 2;
      case Currency.etb:
        return 2;
      case Currency.eur:
        return 2;
      case Currency.fjd:
        return 2;
      case Currency.fkp:
        return 2;
      case Currency.gbp:
        return 2;
      case Currency.gel:
        return 2;
      case Currency.ghs:
        return 2;
      case Currency.gip:
        return 2;
      case Currency.gmd:
        return 2;
      case Currency.gnf:
        return 0;
      case Currency.gtq:
        return 2;
      case Currency.gyd:
        return 2;
      case Currency.hkd:
        return 2;
      case Currency.hnl:
        return 2;
      case Currency.hrk:
        return 2;
      case Currency.htg:
        return 2;
      case Currency.huf:
        return 2;
      case Currency.idr:
        return 2;
      case Currency.ils:
        return 2;
      case Currency.inr:
        return 2;
      case Currency.iqd:
        return 3;
      case Currency.irr:
        return 2;
      case Currency.isk:
        return 0;
      case Currency.jmd:
        return 2;
      case Currency.jod:
        return 3;
      case Currency.jpy:
        return 0;
      case Currency.kes:
        return 2;
      case Currency.kgs:
        return 2;
      case Currency.khr:
        return 2;
      case Currency.kmf:
        return 0;
      case Currency.kpw:
        return 2;
      case Currency.krw:
        return 0;
      case Currency.kwd:
        return 3;
      case Currency.kyd:
        return 2;
      case Currency.kzt:
        return 2;
      case Currency.lak:
        return 2;
      case Currency.lbp:
        return 2;
      case Currency.lkr:
        return 2;
      case Currency.lrd:
        return 2;
      case Currency.lsl:
        return 2;
      case Currency.lyd:
        return 3;
      case Currency.mad:
        return 2;
      case Currency.mdl:
        return 2;
      case Currency.mga:
        return 2;
      case Currency.mkd:
        return 2;
      case Currency.mmk:
        return 2;
      case Currency.mnt:
        return 2;
      case Currency.mop:
        return 2;
      case Currency.mru:
        return 2;
      case Currency.mur:
        return 2;
      case Currency.mvr:
        return 2;
      case Currency.mwk:
        return 2;
      case Currency.mxn:
        return 2;
      case Currency.mxv:
        return 2;
      case Currency.myr:
        return 2;
      case Currency.mzn:
        return 2;
      case Currency.nad:
        return 2;
      case Currency.ngn:
        return 2;
      case Currency.nio:
        return 2;
      case Currency.nok:
        return 2;
      case Currency.npr:
        return 2;
      case Currency.nzd:
        return 2;
      case Currency.omr:
        return 3;
      case Currency.pab:
        return 2;
      case Currency.pen:
        return 2;
      case Currency.pgk:
        return 2;
      case Currency.php:
        return 2;
      case Currency.pkr:
        return 2;
      case Currency.pln:
        return 2;
      case Currency.pyg:
        return 0;
      case Currency.qar:
        return 2;
      case Currency.ron:
        return 2;
      case Currency.rsd:
        return 2;
      case Currency.rub:
        return 2;
      case Currency.rwf:
        return 0;
      case Currency.sar:
        return 2;
      case Currency.sbd:
        return 2;
      case Currency.scr:
        return 2;
      case Currency.sdg:
        return 2;
      case Currency.sek:
        return 2;
      case Currency.sgd:
        return 2;
      case Currency.shp:
        return 2;
      case Currency.sll:
        return 2;
      case Currency.sos:
        return 2;
      case Currency.srd:
        return 2;
      case Currency.ssp:
        return 2;
      case Currency.stn:
        return 2;
      case Currency.svc:
        return 2;
      case Currency.syp:
        return 2;
      case Currency.szl:
        return 2;
      case Currency.thb:
        return 2;
      case Currency.tjs:
        return 2;
      case Currency.tmt:
        return 2;
      case Currency.tnd:
        return 3;
      case Currency.top:
        return 2;
      case Currency.try_:
        return 2;
      case Currency.ttd:
        return 2;
      case Currency.twd:
        return 2;
      case Currency.tzs:
        return 2;
      case Currency.uah:
        return 2;
      case Currency.ugx:
        return 0;
      case Currency.usd:
        return 2;
      case Currency.usn:
        return 2;
      case Currency.uyi:
        return 0;
      case Currency.uyu:
        return 2;
      case Currency.uyw:
        return 4;
      case Currency.uzs:
        return 2;
      case Currency.ves:
        return 2;
      case Currency.vnd:
        return 0;
      case Currency.vuv:
        return 0;
      case Currency.wst:
        return 2;
      case Currency.xaf:
        return 0;
      case Currency.xag:
        return null;
      case Currency.xau:
        return null;
      case Currency.xba:
        return null;
      case Currency.xbb:
        return null;
      case Currency.xbc:
        return null;
      case Currency.xbd:
        return null;
      case Currency.xcd:
        return 2;
      case Currency.xdr:
        return null;
      case Currency.xof:
        return 2;
      case Currency.xpd:
        return null;
      case Currency.xpf:
        return 0;
      case Currency.xpt:
        return null;
      case Currency.xsu:
        return null;
      case Currency.xts:
        return null;
      case Currency.xua:
        return null;
      case Currency.xxx:
        return null;
      case Currency.yer:
        return 2;
      case Currency.zar:
        return 2;
      case Currency.zmw:
        return 2;
      case Currency.zwl:
        return 2;
    }
  }

  /// Return `true` if this currency represents a fund.
  ///
  bool get isFund {
    switch (this) {
      case Currency.aed:
        return false;
      case Currency.afn:
        return false;
      case Currency.all:
        return false;
      case Currency.amd:
        return false;
      case Currency.ang:
        return false;
      case Currency.aoa:
        return false;
      case Currency.ars:
        return false;
      case Currency.aud:
        return false;
      case Currency.awg:
        return false;
      case Currency.azn:
        return false;
      case Currency.bam:
        return false;
      case Currency.bbd:
        return false;
      case Currency.bdt:
        return false;
      case Currency.bgn:
        return false;
      case Currency.bhd:
        return false;
      case Currency.bif:
        return false;
      case Currency.bmd:
        return false;
      case Currency.bnd:
        return false;
      case Currency.bob:
        return false;
      case Currency.bov:
        return true;
      case Currency.brl:
        return false;
      case Currency.bsd:
        return false;
      case Currency.btn:
        return false;
      case Currency.bwp:
        return false;
      case Currency.byn:
        return false;
      case Currency.bzd:
        return false;
      case Currency.cad:
        return false;
      case Currency.cdf:
        return false;
      case Currency.che:
        return true;
      case Currency.chf:
        return false;
      case Currency.chw:
        return true;
      case Currency.clf:
        return true;
      case Currency.clp:
        return false;
      case Currency.cny:
        return false;
      case Currency.cop:
        return false;
      case Currency.cou:
        return true;
      case Currency.crc:
        return false;
      case Currency.cuc:
        return false;
      case Currency.cup:
        return false;
      case Currency.cve:
        return false;
      case Currency.czk:
        return false;
      case Currency.djf:
        return false;
      case Currency.dkk:
        return false;
      case Currency.dop:
        return false;
      case Currency.dzd:
        return false;
      case Currency.egp:
        return false;
      case Currency.ern:
        return false;
      case Currency.etb:
        return false;
      case Currency.eur:
        return false;
      case Currency.fjd:
        return false;
      case Currency.fkp:
        return false;
      case Currency.gbp:
        return false;
      case Currency.gel:
        return false;
      case Currency.ghs:
        return false;
      case Currency.gip:
        return false;
      case Currency.gmd:
        return false;
      case Currency.gnf:
        return false;
      case Currency.gtq:
        return false;
      case Currency.gyd:
        return false;
      case Currency.hkd:
        return false;
      case Currency.hnl:
        return false;
      case Currency.hrk:
        return false;
      case Currency.htg:
        return false;
      case Currency.huf:
        return false;
      case Currency.idr:
        return false;
      case Currency.ils:
        return false;
      case Currency.inr:
        return false;
      case Currency.iqd:
        return false;
      case Currency.irr:
        return false;
      case Currency.isk:
        return false;
      case Currency.jmd:
        return false;
      case Currency.jod:
        return false;
      case Currency.jpy:
        return false;
      case Currency.kes:
        return false;
      case Currency.kgs:
        return false;
      case Currency.khr:
        return false;
      case Currency.kmf:
        return false;
      case Currency.kpw:
        return false;
      case Currency.krw:
        return false;
      case Currency.kwd:
        return false;
      case Currency.kyd:
        return false;
      case Currency.kzt:
        return false;
      case Currency.lak:
        return false;
      case Currency.lbp:
        return false;
      case Currency.lkr:
        return false;
      case Currency.lrd:
        return false;
      case Currency.lsl:
        return false;
      case Currency.lyd:
        return false;
      case Currency.mad:
        return false;
      case Currency.mdl:
        return false;
      case Currency.mga:
        return false;
      case Currency.mkd:
        return false;
      case Currency.mmk:
        return false;
      case Currency.mnt:
        return false;
      case Currency.mop:
        return false;
      case Currency.mru:
        return false;
      case Currency.mur:
        return false;
      case Currency.mvr:
        return false;
      case Currency.mwk:
        return false;
      case Currency.mxn:
        return false;
      case Currency.mxv:
        return true;
      case Currency.myr:
        return false;
      case Currency.mzn:
        return false;
      case Currency.nad:
        return false;
      case Currency.ngn:
        return false;
      case Currency.nio:
        return false;
      case Currency.nok:
        return false;
      case Currency.npr:
        return false;
      case Currency.nzd:
        return false;
      case Currency.omr:
        return false;
      case Currency.pab:
        return false;
      case Currency.pen:
        return false;
      case Currency.pgk:
        return false;
      case Currency.php:
        return false;
      case Currency.pkr:
        return false;
      case Currency.pln:
        return false;
      case Currency.pyg:
        return false;
      case Currency.qar:
        return false;
      case Currency.ron:
        return false;
      case Currency.rsd:
        return false;
      case Currency.rub:
        return false;
      case Currency.rwf:
        return false;
      case Currency.sar:
        return false;
      case Currency.sbd:
        return false;
      case Currency.scr:
        return false;
      case Currency.sdg:
        return false;
      case Currency.sek:
        return false;
      case Currency.sgd:
        return false;
      case Currency.shp:
        return false;
      case Currency.sll:
        return false;
      case Currency.sos:
        return false;
      case Currency.srd:
        return false;
      case Currency.ssp:
        return false;
      case Currency.stn:
        return false;
      case Currency.svc:
        return false;
      case Currency.syp:
        return false;
      case Currency.szl:
        return false;
      case Currency.thb:
        return false;
      case Currency.tjs:
        return false;
      case Currency.tmt:
        return false;
      case Currency.tnd:
        return false;
      case Currency.top:
        return false;
      case Currency.try_:
        return false;
      case Currency.ttd:
        return false;
      case Currency.twd:
        return false;
      case Currency.tzs:
        return false;
      case Currency.uah:
        return false;
      case Currency.ugx:
        return false;
      case Currency.usd:
        return false;
      case Currency.usn:
        return true;
      case Currency.uyi:
        return true;
      case Currency.uyu:
        return false;
      case Currency.uyw:
        return false;
      case Currency.uzs:
        return false;
      case Currency.ves:
        return false;
      case Currency.vnd:
        return false;
      case Currency.vuv:
        return false;
      case Currency.wst:
        return false;
      case Currency.xaf:
        return false;
      case Currency.xag:
        return false;
      case Currency.xau:
        return false;
      case Currency.xba:
        return false;
      case Currency.xbb:
        return false;
      case Currency.xbc:
        return false;
      case Currency.xbd:
        return false;
      case Currency.xcd:
        return false;
      case Currency.xdr:
        return false;
      case Currency.xof:
        return false;
      case Currency.xpd:
        return false;
      case Currency.xpf:
        return false;
      case Currency.xpt:
        return false;
      case Currency.xsu:
        return false;
      case Currency.xts:
        return false;
      case Currency.xua:
        return false;
      case Currency.xxx:
        return false;
      case Currency.yer:
        return false;
      case Currency.zar:
        return false;
      case Currency.zmw:
        return false;
      case Currency.zwl:
        return false;
    }
  }

  /// Gets the name of this currency.
  ///
  String get name {
    switch (this) {
      case Currency.aed:
        return 'UAE Dirham';
      case Currency.afn:
        return 'Afghani';
      case Currency.all:
        return 'Lek';
      case Currency.amd:
        return 'Armenian Dram';
      case Currency.ang:
        return 'Netherlands Antillean Guilder';
      case Currency.aoa:
        return 'Kwanza';
      case Currency.ars:
        return 'Argentine Peso';
      case Currency.aud:
        return 'Australian Dollar';
      case Currency.awg:
        return 'Aruban Florin';
      case Currency.azn:
        return 'Azerbaijan Manat';
      case Currency.bam:
        return 'Convertible Mark';
      case Currency.bbd:
        return 'Barbados Dollar';
      case Currency.bdt:
        return 'Taka';
      case Currency.bgn:
        return 'Bulgarian Lev';
      case Currency.bhd:
        return 'Bahraini Dinar';
      case Currency.bif:
        return 'Burundi Franc';
      case Currency.bmd:
        return 'Bermudian Dollar';
      case Currency.bnd:
        return 'Brunei Dollar';
      case Currency.bob:
        return 'Boliviano';
      case Currency.bov:
        return 'Mvdol';
      case Currency.brl:
        return 'Brazilian Real';
      case Currency.bsd:
        return 'Bahamian Dollar';
      case Currency.btn:
        return 'Ngultrum';
      case Currency.bwp:
        return 'Pula';
      case Currency.byn:
        return 'Belarusian Ruble';
      case Currency.bzd:
        return 'Belize Dollar';
      case Currency.cad:
        return 'Canadian Dollar';
      case Currency.cdf:
        return 'Congolese Franc';
      case Currency.che:
        return 'WIR Euro';
      case Currency.chf:
        return 'Swiss Franc';
      case Currency.chw:
        return 'WIR Franc';
      case Currency.clf:
        return 'Unidad de Fomento';
      case Currency.clp:
        return 'Chilean Peso';
      case Currency.cny:
        return 'Yuan Renminbi';
      case Currency.cop:
        return 'Colombian Peso';
      case Currency.cou:
        return 'Unidad de Valor Real';
      case Currency.crc:
        return 'Costa Rican Colon';
      case Currency.cuc:
        return 'Peso Convertible';
      case Currency.cup:
        return 'Cuban Peso';
      case Currency.cve:
        return 'Cabo Verde Escudo';
      case Currency.czk:
        return 'Czech Koruna';
      case Currency.djf:
        return 'Djibouti Franc';
      case Currency.dkk:
        return 'Danish Krone';
      case Currency.dop:
        return 'Dominican Peso';
      case Currency.dzd:
        return 'Algerian Dinar';
      case Currency.egp:
        return 'Egyptian Pound';
      case Currency.ern:
        return 'Nakfa';
      case Currency.etb:
        return 'Ethiopian Birr';
      case Currency.eur:
        return 'Euro';
      case Currency.fjd:
        return 'Fiji Dollar';
      case Currency.fkp:
        return 'Falkland Islands Pound';
      case Currency.gbp:
        return 'Pound Sterling';
      case Currency.gel:
        return 'Lari';
      case Currency.ghs:
        return 'Ghana Cedi';
      case Currency.gip:
        return 'Gibraltar Pound';
      case Currency.gmd:
        return 'Dalasi';
      case Currency.gnf:
        return 'Guinean Franc';
      case Currency.gtq:
        return 'Quetzal';
      case Currency.gyd:
        return 'Guyana Dollar';
      case Currency.hkd:
        return 'Hong Kong Dollar';
      case Currency.hnl:
        return 'Lempira';
      case Currency.hrk:
        return 'Kuna';
      case Currency.htg:
        return 'Gourde';
      case Currency.huf:
        return 'Forint';
      case Currency.idr:
        return 'Rupiah';
      case Currency.ils:
        return 'New Israeli Sheqel';
      case Currency.inr:
        return 'Indian Rupee';
      case Currency.iqd:
        return 'Iraqi Dinar';
      case Currency.irr:
        return 'Iranian Rial';
      case Currency.isk:
        return 'Iceland Krona';
      case Currency.jmd:
        return 'Jamaican Dollar';
      case Currency.jod:
        return 'Jordanian Dinar';
      case Currency.jpy:
        return 'Yen';
      case Currency.kes:
        return 'Kenyan Shilling';
      case Currency.kgs:
        return 'Som';
      case Currency.khr:
        return 'Riel';
      case Currency.kmf:
        return 'Comorian Franc';
      case Currency.kpw:
        return 'North Korean Won';
      case Currency.krw:
        return 'Won';
      case Currency.kwd:
        return 'Kuwaiti Dinar';
      case Currency.kyd:
        return 'Cayman Islands Dollar';
      case Currency.kzt:
        return 'Tenge';
      case Currency.lak:
        return 'Lao Kip';
      case Currency.lbp:
        return 'Lebanese Pound';
      case Currency.lkr:
        return 'Sri Lanka Rupee';
      case Currency.lrd:
        return 'Liberian Dollar';
      case Currency.lsl:
        return 'Loti';
      case Currency.lyd:
        return 'Libyan Dinar';
      case Currency.mad:
        return 'Moroccan Dirham';
      case Currency.mdl:
        return 'Moldovan Leu';
      case Currency.mga:
        return 'Malagasy Ariary';
      case Currency.mkd:
        return 'Denar';
      case Currency.mmk:
        return 'Kyat';
      case Currency.mnt:
        return 'Tugrik';
      case Currency.mop:
        return 'Pataca';
      case Currency.mru:
        return 'Ouguiya';
      case Currency.mur:
        return 'Mauritius Rupee';
      case Currency.mvr:
        return 'Rufiyaa';
      case Currency.mwk:
        return 'Malawi Kwacha';
      case Currency.mxn:
        return 'Mexican Peso';
      case Currency.mxv:
        return 'Mexican Unidad de Inversion (UDI)';
      case Currency.myr:
        return 'Malaysian Ringgit';
      case Currency.mzn:
        return 'Mozambique Metical';
      case Currency.nad:
        return 'Namibia Dollar';
      case Currency.ngn:
        return 'Naira';
      case Currency.nio:
        return 'Cordoba Oro';
      case Currency.nok:
        return 'Norwegian Krone';
      case Currency.npr:
        return 'Nepalese Rupee';
      case Currency.nzd:
        return 'New Zealand Dollar';
      case Currency.omr:
        return 'Rial Omani';
      case Currency.pab:
        return 'Balboa';
      case Currency.pen:
        return 'Sol';
      case Currency.pgk:
        return 'Kina';
      case Currency.php:
        return 'Philippine Peso';
      case Currency.pkr:
        return 'Pakistan Rupee';
      case Currency.pln:
        return 'Zloty';
      case Currency.pyg:
        return 'Guarani';
      case Currency.qar:
        return 'Qatari Rial';
      case Currency.ron:
        return 'Romanian Leu';
      case Currency.rsd:
        return 'Serbian Dinar';
      case Currency.rub:
        return 'Russian Ruble';
      case Currency.rwf:
        return 'Rwanda Franc';
      case Currency.sar:
        return 'Saudi Riyal';
      case Currency.sbd:
        return 'Solomon Islands Dollar';
      case Currency.scr:
        return 'Seychelles Rupee';
      case Currency.sdg:
        return 'Sudanese Pound';
      case Currency.sek:
        return 'Swedish Krona';
      case Currency.sgd:
        return 'Singapore Dollar';
      case Currency.shp:
        return 'Saint Helena Pound';
      case Currency.sll:
        return 'Leone';
      case Currency.sos:
        return 'Somali Shilling';
      case Currency.srd:
        return 'Surinam Dollar';
      case Currency.ssp:
        return 'South Sudanese Pound';
      case Currency.stn:
        return 'Dobra';
      case Currency.svc:
        return 'El Salvador Colon';
      case Currency.syp:
        return 'Syrian Pound';
      case Currency.szl:
        return 'Lilangeni';
      case Currency.thb:
        return 'Baht';
      case Currency.tjs:
        return 'Somoni';
      case Currency.tmt:
        return 'Turkmenistan New Manat';
      case Currency.tnd:
        return 'Tunisian Dinar';
      case Currency.top:
        return 'Pa’anga';
      case Currency.try_:
        return 'Turkish Lira';
      case Currency.ttd:
        return 'Trinidad and Tobago Dollar';
      case Currency.twd:
        return 'New Taiwan Dollar';
      case Currency.tzs:
        return 'Tanzanian Shilling';
      case Currency.uah:
        return 'Hryvnia';
      case Currency.ugx:
        return 'Uganda Shilling';
      case Currency.usd:
        return 'US Dollar';
      case Currency.usn:
        return 'US Dollar (Next day)';
      case Currency.uyi:
        return 'Uruguay Peso en Unidades Indexadas (UI)';
      case Currency.uyu:
        return 'Peso Uruguayo';
      case Currency.uyw:
        return 'Unidad Previsional';
      case Currency.uzs:
        return 'Uzbekistan Sum';
      case Currency.ves:
        return 'Bolívar Soberano';
      case Currency.vnd:
        return 'Dong';
      case Currency.vuv:
        return 'Vatu';
      case Currency.wst:
        return 'Tala';
      case Currency.xaf:
        return 'CFA Franc BEAC';
      case Currency.xag:
        return 'Silver';
      case Currency.xau:
        return 'Gold';
      case Currency.xba:
        return 'Bond Markets Unit European Composite Unit (EURCO)';
      case Currency.xbb:
        return 'Bond Markets Unit European Monetary Unit (E.M.U.-6)';
      case Currency.xbc:
        return 'Bond Markets Unit European Unit of Account 9 (E.U.A.-9)';
      case Currency.xbd:
        return 'Bond Markets Unit European Unit of Account 17 (E.U.A.-17)';
      case Currency.xcd:
        return 'East Caribbean Dollar';
      case Currency.xdr:
        return 'SDR (Special Drawing Right)';
      case Currency.xof:
        return 'CFA Franc BCEAO';
      case Currency.xpd:
        return 'Palladium';
      case Currency.xpf:
        return 'CFP Franc';
      case Currency.xpt:
        return 'Platinum';
      case Currency.xsu:
        return 'Sucre';
      case Currency.xts:
        return 'Codes specifically reserved for testing purposes';
      case Currency.xua:
        return 'ADB Unit of Account';
      case Currency.xxx:
        return 'The codes assigned for transactions where no currency is involved';
      case Currency.yer:
        return 'Yemeni Rial';
      case Currency.zar:
        return 'Rand';
      case Currency.zmw:
        return 'Zambian Kwacha';
      case Currency.zwl:
        return 'Zimbabwe Dollar';
    }
  }

  /// Gets the numerical code representation for this currency.
  ///
  String get numerical {
    switch (this) {
      case Currency.aed:
        return '784';
      case Currency.afn:
        return '971';
      case Currency.all:
        return '008';
      case Currency.amd:
        return '051';
      case Currency.ang:
        return '532';
      case Currency.aoa:
        return '973';
      case Currency.ars:
        return '032';
      case Currency.aud:
        return '036';
      case Currency.awg:
        return '533';
      case Currency.azn:
        return '944';
      case Currency.bam:
        return '977';
      case Currency.bbd:
        return '052';
      case Currency.bdt:
        return '050';
      case Currency.bgn:
        return '975';
      case Currency.bhd:
        return '048';
      case Currency.bif:
        return '108';
      case Currency.bmd:
        return '060';
      case Currency.bnd:
        return '096';
      case Currency.bob:
        return '068';
      case Currency.bov:
        return '984';
      case Currency.brl:
        return '986';
      case Currency.bsd:
        return '044';
      case Currency.btn:
        return '064';
      case Currency.bwp:
        return '072';
      case Currency.byn:
        return '933';
      case Currency.bzd:
        return '084';
      case Currency.cad:
        return '124';
      case Currency.cdf:
        return '976';
      case Currency.che:
        return '947';
      case Currency.chf:
        return '756';
      case Currency.chw:
        return '948';
      case Currency.clf:
        return '990';
      case Currency.clp:
        return '152';
      case Currency.cny:
        return '156';
      case Currency.cop:
        return '170';
      case Currency.cou:
        return '970';
      case Currency.crc:
        return '188';
      case Currency.cuc:
        return '931';
      case Currency.cup:
        return '192';
      case Currency.cve:
        return '132';
      case Currency.czk:
        return '203';
      case Currency.djf:
        return '262';
      case Currency.dkk:
        return '208';
      case Currency.dop:
        return '214';
      case Currency.dzd:
        return '012';
      case Currency.egp:
        return '818';
      case Currency.ern:
        return '232';
      case Currency.etb:
        return '230';
      case Currency.eur:
        return '978';
      case Currency.fjd:
        return '242';
      case Currency.fkp:
        return '238';
      case Currency.gbp:
        return '826';
      case Currency.gel:
        return '981';
      case Currency.ghs:
        return '936';
      case Currency.gip:
        return '292';
      case Currency.gmd:
        return '270';
      case Currency.gnf:
        return '324';
      case Currency.gtq:
        return '320';
      case Currency.gyd:
        return '328';
      case Currency.hkd:
        return '344';
      case Currency.hnl:
        return '340';
      case Currency.hrk:
        return '191';
      case Currency.htg:
        return '332';
      case Currency.huf:
        return '348';
      case Currency.idr:
        return '360';
      case Currency.ils:
        return '376';
      case Currency.inr:
        return '356';
      case Currency.iqd:
        return '368';
      case Currency.irr:
        return '364';
      case Currency.isk:
        return '352';
      case Currency.jmd:
        return '388';
      case Currency.jod:
        return '400';
      case Currency.jpy:
        return '392';
      case Currency.kes:
        return '404';
      case Currency.kgs:
        return '417';
      case Currency.khr:
        return '116';
      case Currency.kmf:
        return '174';
      case Currency.kpw:
        return '408';
      case Currency.krw:
        return '410';
      case Currency.kwd:
        return '414';
      case Currency.kyd:
        return '136';
      case Currency.kzt:
        return '398';
      case Currency.lak:
        return '418';
      case Currency.lbp:
        return '422';
      case Currency.lkr:
        return '144';
      case Currency.lrd:
        return '430';
      case Currency.lsl:
        return '426';
      case Currency.lyd:
        return '434';
      case Currency.mad:
        return '504';
      case Currency.mdl:
        return '498';
      case Currency.mga:
        return '969';
      case Currency.mkd:
        return '807';
      case Currency.mmk:
        return '104';
      case Currency.mnt:
        return '496';
      case Currency.mop:
        return '446';
      case Currency.mru:
        return '929';
      case Currency.mur:
        return '480';
      case Currency.mvr:
        return '462';
      case Currency.mwk:
        return '454';
      case Currency.mxn:
        return '484';
      case Currency.mxv:
        return '979';
      case Currency.myr:
        return '458';
      case Currency.mzn:
        return '943';
      case Currency.nad:
        return '516';
      case Currency.ngn:
        return '566';
      case Currency.nio:
        return '558';
      case Currency.nok:
        return '578';
      case Currency.npr:
        return '524';
      case Currency.nzd:
        return '554';
      case Currency.omr:
        return '512';
      case Currency.pab:
        return '590';
      case Currency.pen:
        return '604';
      case Currency.pgk:
        return '598';
      case Currency.php:
        return '608';
      case Currency.pkr:
        return '586';
      case Currency.pln:
        return '985';
      case Currency.pyg:
        return '600';
      case Currency.qar:
        return '634';
      case Currency.ron:
        return '946';
      case Currency.rsd:
        return '941';
      case Currency.rub:
        return '643';
      case Currency.rwf:
        return '646';
      case Currency.sar:
        return '682';
      case Currency.sbd:
        return '090';
      case Currency.scr:
        return '690';
      case Currency.sdg:
        return '938';
      case Currency.sek:
        return '752';
      case Currency.sgd:
        return '702';
      case Currency.shp:
        return '654';
      case Currency.sll:
        return '694';
      case Currency.sos:
        return '706';
      case Currency.srd:
        return '968';
      case Currency.ssp:
        return '728';
      case Currency.stn:
        return '930';
      case Currency.svc:
        return '222';
      case Currency.syp:
        return '760';
      case Currency.szl:
        return '748';
      case Currency.thb:
        return '764';
      case Currency.tjs:
        return '972';
      case Currency.tmt:
        return '934';
      case Currency.tnd:
        return '788';
      case Currency.top:
        return '776';
      case Currency.try_:
        return '949';
      case Currency.ttd:
        return '780';
      case Currency.twd:
        return '901';
      case Currency.tzs:
        return '834';
      case Currency.uah:
        return '980';
      case Currency.ugx:
        return '800';
      case Currency.usd:
        return '840';
      case Currency.usn:
        return '997';
      case Currency.uyi:
        return '940';
      case Currency.uyu:
        return '858';
      case Currency.uyw:
        return '927';
      case Currency.uzs:
        return '860';
      case Currency.ves:
        return '928';
      case Currency.vnd:
        return '704';
      case Currency.vuv:
        return '548';
      case Currency.wst:
        return '882';
      case Currency.xaf:
        return '950';
      case Currency.xag:
        return '961';
      case Currency.xau:
        return '959';
      case Currency.xba:
        return '955';
      case Currency.xbb:
        return '956';
      case Currency.xbc:
        return '957';
      case Currency.xbd:
        return '958';
      case Currency.xcd:
        return '951';
      case Currency.xdr:
        return '960';
      case Currency.xof:
        return '952';
      case Currency.xpd:
        return '964';
      case Currency.xpf:
        return '953';
      case Currency.xpt:
        return '962';
      case Currency.xsu:
        return '994';
      case Currency.xts:
        return '963';
      case Currency.xua:
        return '965';
      case Currency.xxx:
        return '999';
      case Currency.yer:
        return '886';
      case Currency.zar:
        return '710';
      case Currency.zmw:
        return '967';
      case Currency.zwl:
        return '932';
    }
  }

  /// Gets the corresponding symbol for this currency.
  ///
  String get symbol {
    switch (this) {
      case Currency.aed:
        return 'د.إ';
      case Currency.afn:
        return '؋';
      case Currency.all:
        return 'L';
      case Currency.amd:
        return '֏';
      case Currency.ang:
        return 'ƒ';
      case Currency.aoa:
        return 'Kz';
      case Currency.ars:
        return r'$';
      case Currency.aud:
        return r'$';
      case Currency.awg:
        return 'ƒ';
      case Currency.azn:
        return '₼';
      case Currency.bam:
        return 'KM';
      case Currency.bbd:
        return r'$';
      case Currency.bdt:
        return '৳';
      case Currency.bgn:
        return 'лв.';
      case Currency.bhd:
        return 'د.ب';
      case Currency.bif:
        return 'FBu';
      case Currency.bmd:
        return r'$';
      case Currency.bnd:
        return r'$';
      case Currency.bob:
        return 'Bs';
      case Currency.bov:
        return r'$';
      case Currency.brl:
        return r'R$';
      case Currency.bsd:
        return r'B$';
      case Currency.btn:
        return 'Nu.';
      case Currency.bwp:
        return 'P';
      case Currency.byn:
        return 'BYN';
      case Currency.bzd:
        return r'$';
      case Currency.cad:
        return r'$';
      case Currency.cdf:
        return 'FC';
      case Currency.che:
        return '';
      case Currency.chf:
        return 'CHF';
      case Currency.chw:
        return '';
      case Currency.clf:
        return 'UF';
      case Currency.clp:
        return r'$';
      case Currency.cny:
        return '¥';
      case Currency.cop:
        return r'$';
      case Currency.cou:
        return 'UVR';
      case Currency.crc:
        return '₡';
      case Currency.cuc:
        return r'$';
      case Currency.cup:
        return '₱';
      case Currency.cve:
        return r'$';
      case Currency.czk:
        return 'Kč';
      case Currency.djf:
        return 'Fdj';
      case Currency.dkk:
        return 'Kr.';
      case Currency.dop:
        return r'$';
      case Currency.dzd:
        return 'دج';
      case Currency.egp:
        return '£';
      case Currency.ern:
        return 'Nkf';
      case Currency.etb:
        return 'ብር';
      case Currency.eur:
        return '€';
      case Currency.fjd:
        return r'FJ$';
      case Currency.fkp:
        return '£';
      case Currency.gbp:
        return '£';
      case Currency.gel:
        return 'ლ';
      case Currency.ghs:
        return 'GH₵';
      case Currency.gip:
        return '£';
      case Currency.gmd:
        return 'D';
      case Currency.gnf:
        return 'FG';
      case Currency.gtq:
        return 'Q';
      case Currency.gyd:
        return r'$';
      case Currency.hkd:
        return r'$';
      case Currency.hnl:
        return 'L';
      case Currency.hrk:
        return 'kn';
      case Currency.htg:
        return 'G';
      case Currency.huf:
        return 'Ft';
      case Currency.idr:
        return 'Rp';
      case Currency.ils:
        return '₪';
      case Currency.inr:
        return '₹';
      case Currency.iqd:
        return 'ع.د';
      case Currency.irr:
        return '﷼';
      case Currency.isk:
        return 'kr';
      case Currency.jmd:
        return r'$';
      case Currency.jod:
        return 'د.ا';
      case Currency.jpy:
        return '¥';
      case Currency.kes:
        return 'KSh';
      case Currency.kgs:
        return 'лв';
      case Currency.khr:
        return '៛';
      case Currency.kmf:
        return 'CF';
      case Currency.kpw:
        return '₩';
      case Currency.krw:
        return '₩';
      case Currency.kwd:
        return 'د.ك';
      case Currency.kyd:
        return r'$';
      case Currency.kzt:
        return '₸';
      case Currency.lak:
        return '₭';
      case Currency.lbp:
        return 'ل.ل.';
      case Currency.lkr:
        return '₨';
      case Currency.lrd:
        return r'L$';
      case Currency.lsl:
        return 'L';
      case Currency.lyd:
        return 'ل.د';
      case Currency.mad:
        return 'DH';
      case Currency.mdl:
        return 'L';
      case Currency.mga:
        return 'Ar';
      case Currency.mkd:
        return 'Ден';
      case Currency.mmk:
        return 'K';
      case Currency.mnt:
        return '₮';
      case Currency.mop:
        return r'MOP$';
      case Currency.mru:
        return 'UM';
      case Currency.mur:
        return '₨';
      case Currency.mvr:
        return 'ރ.';
      case Currency.mwk:
        return 'MK';
      case Currency.mxn:
        return r'$';
      case Currency.mxv:
        return '';
      case Currency.myr:
        return 'RM';
      case Currency.mzn:
        return 'MT';
      case Currency.nad:
        return r'N$';
      case Currency.ngn:
        return '₦';
      case Currency.nio:
        return r'C$';
      case Currency.nok:
        return 'kr';
      case Currency.npr:
        return '₨';
      case Currency.nzd:
        return r'$';
      case Currency.omr:
        return 'ر.ع.';
      case Currency.pab:
        return 'B/.';
      case Currency.pen:
        return 'S/.';
      case Currency.pgk:
        return 'K';
      case Currency.php:
        return '₱';
      case Currency.pkr:
        return '₨';
      case Currency.pln:
        return 'zł';
      case Currency.pyg:
        return '₲';
      case Currency.qar:
        return '﷼';
      case Currency.ron:
        return 'lei';
      case Currency.rsd:
        return 'Дин.';
      case Currency.rub:
        return '₽';
      case Currency.rwf:
        return 'FRw';
      case Currency.sar:
        return '﷼';
      case Currency.sbd:
        return r'$';
      case Currency.scr:
        return '₨';
      case Currency.sdg:
        return 'ج.س.';
      case Currency.sek:
        return 'kr';
      case Currency.sgd:
        return r'$';
      case Currency.shp:
        return '£';
      case Currency.sll:
        return 'Le';
      case Currency.sos:
        return 'Sh.So.';
      case Currency.srd:
        return r'$';
      case Currency.ssp:
        return '£';
      case Currency.stn:
        return 'Db';
      case Currency.svc:
        return '₡';
      case Currency.syp:
        return 'LS';
      case Currency.szl:
        return 'E';
      case Currency.thb:
        return '฿';
      case Currency.tjs:
        return 'SM';
      case Currency.tmt:
        return 'T';
      case Currency.tnd:
        return 'د.ت';
      case Currency.top:
        return r'T$';
      case Currency.try_:
        return '₺';
      case Currency.ttd:
        return r'TT$';
      case Currency.twd:
        return r'NT$';
      case Currency.tzs:
        return 'TSh';
      case Currency.uah:
        return '₴';
      case Currency.ugx:
        return 'USh';
      case Currency.usd:
        return r'$';
      case Currency.usn:
        return 'USN';
      case Currency.uyi:
        return '';
      case Currency.uyu:
        return r'$';
      case Currency.uyw:
        return '';
      case Currency.uzs:
        return 'so’m';
      case Currency.ves:
        return 'Bs.S.';
      case Currency.vnd:
        return '₫';
      case Currency.vuv:
        return 'VT';
      case Currency.wst:
        return r'WS$';
      case Currency.xaf:
        return 'FCFA';
      case Currency.xag:
        return '';
      case Currency.xau:
        return '';
      case Currency.xba:
        return '';
      case Currency.xbb:
        return '';
      case Currency.xbc:
        return '';
      case Currency.xbd:
        return '';
      case Currency.xcd:
        return r'$';
      case Currency.xdr:
        return '';
      case Currency.xof:
        return 'CFA';
      case Currency.xpd:
        return '';
      case Currency.xpf:
        return '₣';
      case Currency.xpt:
        return '';
      case Currency.xsu:
        return '';
      case Currency.xts:
        return '';
      case Currency.xua:
        return '';
      case Currency.xxx:
        return '';
      case Currency.yer:
        return '﷼';
      case Currency.zar:
        return 'R';
      case Currency.zmw:
        return 'ZK';
      case Currency.zwl:
        return r'$';
    }
  }

  /// Returns the corresponding alphabetic value of this `Currency` value.
  ///
  String string() => alphabetic;
}

/// This extension provides useful tools in order to convert a `String` into a
/// `Currency` value.
///
extension StringToCurrencyX on String {
  /// Converts this string into the corresponding `Currency` value.
  ///
  Currency toCurrency() {
    for (final Currency currency in Currency.values) {
      if (toUpperCase() == currency.alphabetic || this == currency.numerical)
        return currency;
    }

    throw FormatException(
        'The string does not contains a valid (or unique) Currency'
        ' representation.');
  }
}
